local L = RaidframeProfileSwitcherLocalization

SLASH_RPS1, SLASH_RPS2 = "/rps", "/raidframeprofileswitcher"
function SlashCmdList.RPS(msg, editbox)
  if msg == "" or msg == "help" then
    chatInfo(L.HELP_COMMANDS)
    chatInfo(L.HELP_DISPLAY)
    chatInfo(L.HELP_PARTY)
    chatInfo(L.HELP_RAID)
    chatInfo(L.HELP_BIGRAID)
    chatInfo(L.HELP_RESET)
  elseif msg == "reset" then
    resetRaidProfiles(true)
  elseif msg == "display" then
    if RPSVars == nil or type(RPSVars) ~= "table" then resetRaidProfiles(false) end
    chatInfo(string.format(L.DISPLAY_PROFILES, RPSVars["PARTY"], RPSVars["RAID"], RPSVars["BIGRAID"]))
  else
    local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
    if args ~= "" then
      if cmd == "party" then
        setNewRaidframeProfile(args, "PARTY")
      elseif cmd == "raid" then
        setNewRaidframeProfile(args, "RAID")
      elseif cmd == "bigraid" then
        setNewRaidframeProfile(args, "BIGRAID")
      else
        chatWarning(string.format(L.SET_SETTING_NA, cmd))
      end
    else
      chatWarning(L.SET_SETTING_MISSING)
    end
  end
end

function setNewRaidframeProfile(profileName, settingName)
  if RaidProfileExists(profileName) then
    RPSVars[settingName] = profileName
    chatSuccess(string.format(L.PROFILE_SET, profileName, settingName:lower()))
  else
    chatWarning(string.format(L.PROFILE_SET_NA, profileName))
  end
end

function onEvent()
  if RPSVars == nil or type(RPSVars) ~= "table" then resetRaidProfiles(false) end
  if InCombatLockdown() == false then
    maxGroupNumber = getMaxGroupNumber()
    activeRaidProfile = GetActiveRaidProfile()
    targetRaidProfile = ""
    if InActiveBattlefield() or maxGroupNumber > 5 then
      targetRaidProfile = RPSVars["BIGRAID"]
    elseif maxGroupNumber > 1 then
      targetRaidProfile = RPSVars["RAID"]
    else
      targetRaidProfile = RPSVars["PARTY"]
    end
    if activeRaidProfile ~= targetRaidProfile then
      if RaidProfileExists(targetRaidProfile) then
        CompactUnitFrameProfiles_ActivateRaidProfile(targetRaidProfile);
        chatSuccess(string.format(L.PROFILE_SWITCHED, targetRaidProfile, maxGroupNumber))
      else
        chatWarning(string.format(L.PROFILE_SWITCHING_FAILED, targetRaidProfile))
      end
    end
  end
end

function getMaxGroupNumber()
  maxGroupNumber = 1
  for i=1,GetNumGroupMembers() do local n,_,g=GetRaidRosterInfo(i)
    if g == 8 then
      return 8
    elseif g > maxGroupNumber then
      maxGroupNumber = g
    end
  end 
  return maxGroupNumber
end

function resetRaidProfiles(withMsg)
  RPSVars = {}
  RPSVars["PARTY"] = "Dungeon"
  RPSVars["RAID"] = "Raid"
  RPSVars["BIGRAID"] = "BigRaid"
  if withMsg then chatSuccess(L.PROFILES_RESET) end
end

function chatInfo(txt)
  ChatFrame1:AddMessage(txt, 1, 1, 0)
end

function chatWarning(txt)
  ChatFrame1:AddMessage(txt, 1, 0, 0)
end

function chatSuccess(txt)
  ChatFrame1:AddMessage(txt, 0, 1, 0)
end

local frameUpdate = CreateFrame("Frame")
frameUpdate:RegisterEvent("GROUP_ROSTER_UPDATE")
frameUpdate:RegisterEvent("PLAYER_REGEN_ENABLED")
frameUpdate:SetScript("OnEvent", onEvent);
